from openfisca_us.model_api import *


class in_agi(Variable):
    value_type = float
    entity = TaxUnit
    label = "IN adjusted gross income"
    unit = USD
    definition_period = YEAR
    reference = (
        "http://iga.in.gov/legislative/laws/2021/ic/titles/006#6-3-1-3.5"
    )
    defined_for = StateCode.IN

    def formula(tax_unit, period, parameters):
        federal_agi = tax_unit("adjusted_gross_income", period)
        in_add_backs = tax_unit("in_add_backs", period)
        in_deductions = tax_unit("in_deductions", period)
        in_exemptions = tax_unit("in_exemptions", period)

        return federal_agi + in_add_backs - in_deductions - in_exemptions



from openfisca_us.model_api import *


class maximum_qbid(Variable):
    value_type = float
    entity = TaxUnit
    label = "Maximum qualified business income deduction"
    unit = USD
    definition_period = YEAR
    reference = "https://www.law.cornell.edu/uscode/text/26/199A#b_2"

    def formula(tax_unit, period, parameters):
        qbi = add(tax_unit, period, ["qualified_business_income"])
        qbid = parameters(period).gov.irs.deductions.qbi
        return qbid.max.rate * qbi



from openfisca_us.model_api import *


class iitax(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    unit = USD
    label = "Federal income tax"
    documentation = "Total federal individual income tax liability."

    def formula(tax_unit, period, parameters):
        before_refundable_credits = tax_unit(
            "income_tax_before_refundable_credits", period
        )
        refundable_credits = tax_unit("income_tax_refundable_credits", period)
        return before_refundable_credits - refundable_credits


income_tax = variable_alias("income_tax", iitax)



from openfisca_us.model_api import *


class md_refundable_eitc(Variable):
    value_type = float
    entity = TaxUnit
    label = "MD EITC refundable State tax credit"
    unit = USD
    documentation = "Refundable EITC credit reducing MD State income tax."
    definition_period = YEAR
    reference = "https://casetext.com/statute/code-of-maryland/article-tax-general/title-10-income-tax/subtitle-7-income-tax-credits/section-10-704-effective-until-6302023-for-earned-income"
    defined_for = StateCode.MD

    def formula(tax_unit, period, parameters):
        non_refundable_eitc = tax_unit("md_non_refundable_eitc", period)
        md_eitc = parameters(period).gov.states.md.tax.income.credits.eitc
        income_tax = tax_unit("md_income_tax_before_credits", period)
        excess = max_(0, non_refundable_eitc - income_tax)
        childless = tax_unit("eitc_child_count", period) == 0
        return where(
            childless,
            excess,
            md_eitc.refundable_match * excess,
        )



from openfisca_us.model_api import *


class social_security_reported(Variable):
    value_type = float
    entity = Person
    label = "Social Security (reported)"
    unit = USD
    definition_period = YEAR



from openfisca_us.model_api import *


class nonbusiness_energy_property_credit(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Nonbusiness energy property credit"
    unit = USD
    reference = "https://www.law.cornell.edu/uscode/text/26/25C"

    def formula(tax_unit, period, parameters):
        p = parameters(period).gov.irs.credits.residential_energy.nonbusiness
        if not p.in_effect:
            return 0
        improvements = tax_unit(
            "qualified_energy_efficiency_improvements_expenditures", period
        )
        improvements_credit = improvements * p.improvements.rate
        # Full property expenditures count for the credit.
        property_credit = tax_unit(
            "capped_residential_energy_property_expenditures", period
        )
        uncapped_credit = improvements_credit + property_credit
        # Apply lifetime limitation.
        prior_credits = tax_unit(
            "prior_nonbusiness_energy_property_credits", period
        )
        remaining_credit = p.cap.lifetime.total - prior_credits
        return min_(remaining_credit, uncapped_credit)



from openfisca_us.model_api import *


class residential_energy_credit(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Residential energy credit from Form 5695"
    unit = USD

    formula = sum_of_variables(
        [
            "residential_energy_efficient_property_credit",
            "nonbusiness_energy_property_credit",
        ]
    )



from openfisca_us.model_api import *


class capped_residential_energy_property_expenditures(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Capped residential energy property expenditures"
    unit = USD
    reference = "https://www.law.cornell.edu/uscode/text/26/25C#b_3"

    formula = sum_of_variables(
        [
            "capped_advanced_main_air_circulating_fan_expenditures",
            "capped_energy_efficient_building_property_expenditures",
            "capped_qualified_furnace_or_hot_water_boiler_expenditures",
        ]
    )



from openfisca_us.model_api import *


class mo_income_tax_before_credits(Variable):
    value_type = float
    entity = TaxUnit
    label = "MO income tax before credits"
    unit = USD
    definition_period = YEAR
    reference = "https://dor.mo.gov/forms/MO-1040%20Print%20Only_2021.pdf"
    defined_for = StateCode.MO

    def formula(tax_unit, period, parameters):
        taxable_income = tax_unit("mo_taxable_income", period)
        rates = parameters(period).gov.states.mo.tax.income.rates
        return rates.calc(taxable_income)



from openfisca_us.model_api import *


class standard(Variable):
    value_type = float
    entity = TaxUnit
    label = "Standard deduction (zero for itemizers)"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        # Calculate basic standard deduction
        basic_stded = tax_unit("basic_standard_deduction", period)
        filing_status = tax_unit("filing_status", period)
        separate_filer_itemizes = tax_unit("separate_filer_itemizes", period)
        filing_statuses = filing_status.possible_values

        # Calculate extra standard deduction for aged and blind
        extra_stded = tax_unit("aged_blind_extra_standard_deduction", period)

        # Calculate the total standard deduction.
        initial_standard = basic_stded + extra_stded
        # Separate filers get zero if their spouse itemizes.
        return where(
            (filing_status == filing_statuses.SEPARATE)
            & separate_filer_itemizes,
            0,
            initial_standard,
        )



from openfisca_us.model_api import *
from openfisca_us import CountryTaxBenefitSystem


class dividend_income(Variable):
    value_type = float
    entity = Person
    label = "Dividend income"
    unit = USD
    definition_period = YEAR

    def formula(person, period, parameters):
        simulation = person.simulation
        qualified_dividends = simulation.get_array(
            "qualified_dividend_income", period
        )
        non_qualified_dividends = simulation.get_array(
            "non_qualified_dividend_income", period
        )
        zeros = np.zeros(person.count)
        if qualified_dividends is None and non_qualified_dividends is None:
            return zeros
        if qualified_dividends is None:
            qualified_dividends = zeros
        if non_qualified_dividends is None:
            non_qualified_dividends = zeros
        return qualified_dividends + non_qualified_dividends



from openfisca_us.model_api import *


class ca_eitc(Variable):
    value_type = float
    entity = TaxUnit
    label = "CalEITC amount"
    unit = USD
    definition_period = YEAR
    reference = "https://www.ftb.ca.gov/file/personal/credits/california-earned-income-tax-credit.html#What-you-ll-get"



from openfisca_us.model_api import *


class non_refundable_ctc(Variable):
    value_type = float
    entity = TaxUnit
    label = "Non-refundable CTC"
    unit = USD
    documentation = (
        "The portion of the Child Tax Credit that is not refundable."
    )
    definition_period = YEAR

    def formula(tax_unit, period, parameters):
        return tax_unit("ctc", period) - tax_unit("refundable_ctc", period)



from openfisca_us.model_api import *


class capped_qualified_furnace_or_hot_water_boiler_expenditures(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Capped qualified furnace or hot water boiler expenditures"
    unit = USD
    reference = "https://www.law.cornell.edu/uscode/text/26/25C#b_3_B"

    def formula(tax_unit, period, parameters):
        uncapped = tax_unit(
            "qualified_furnace_or_hot_water_boiler_expenditures", period
        )
        p = parameters(period).gov.irs.credits.residential_energy.nonbusiness
        return min_(uncapped, p.cap.qualified_furnace_or_hot_water_boiler)



from openfisca_us.model_api import *


class tanf(Variable):
    value_type = float
    entity = SPMUnit
    definition_period = YEAR
    label = "TANF"
    documentation = (
        "Amount of Temporary Assistance for Needy Families benefit received."
    )
    unit = USD

    def formula(spm_unit, period, parameters):
        tanf_reported = add(spm_unit, period, ["tanf_reported"])
        if tanf_reported.sum() > 0:
            return tanf_reported
        # Obtain eligibility.
        eligible = spm_unit("is_tanf_eligible", period)
        # Obtain amount they would receive if they were eligible.
        amount_if_eligible = spm_unit("tanf_amount_if_eligible", period)
        return where(eligible, amount_if_eligible, 0)



from openfisca_us.model_api import *


class taxsim_siitax(Variable):
    value_type = float
    entity = TaxUnit
    label = "State income tax liability"
    unit = USD
    definition_period = YEAR

    formula = sum_of_variables(["state_income_tax"])



from openfisca_us.model_api import *


class energy_efficient_building_property_expenditures(Variable):
    value_type = float
    entity = TaxUnit
    label = "Energy efficient building property expenditures"
    documentation = "Expenditures on energy-efficient property, including electric heat pump water heaters, electric heat pumps, central air conditioners, and natural gas, propane, or oil water heaters."
    unit = USD
    definition_period = YEAR
    reference = "https://www.law.cornell.edu/uscode/text/26/25C#d_3"



from openfisca_us.model_api import *


class md_deductions(Variable):
    value_type = float
    entity = TaxUnit
    label = "MD deductions"
    unit = USD
    definition_period = YEAR
    reference = "https://govt.westlaw.com/mdc/Document/N05479690A64A11DBB5DDAC3692B918BC?viewType=FullText&originationContext=documenttoc&transitionType=CategoryPageItem&contextData=(sc.Default)"
    defined_for = StateCode.MD

    def formula(tax_unit, period, parameters):
        # Check if the tax_unit itemized on their federal returns:
        tax_unit_itemizes = tax_unit("tax_unit_itemizes", period)
        standard_deduction = tax_unit("md_standard_deduction", period)
        gov = parameters(period).gov
        federal_deductions_if_itemizing = (
            gov.irs.deductions.deductions_if_itemizing
        )
        federal_deductions_if_itemizing = [
            deduction
            for deduction in federal_deductions_if_itemizing
            if deduction
            not in [
                "salt_deduction",
                "qualified_business_income_deduction",
            ]
        ]
        itemized_deductions_less_salt = add(
            tax_unit, period, federal_deductions_if_itemizing
        )
        return where(
            tax_unit_itemizes,
            itemized_deductions_less_salt,
            standard_deduction,
        )



from openfisca_us.model_api import *


class ut_total_income(Variable):
    value_type = float
    entity = TaxUnit
    label = "UT total income"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.UT

    def formula(tax_unit, period, parameters):
        federal_agi = tax_unit("adjusted_gross_income", period)
        ut_additions = tax_unit("ut_additions_to_income", period)

        return federal_agi + ut_additions



from openfisca_us.model_api import *


class ma_income_tax(Variable):
    value_type = float
    entity = TaxUnit
    label = "MA income tax"
    unit = USD
    documentation = "Massachusetts State income tax."
    definition_period = YEAR
    reference = "https://www.mass.gov/doc/2021-form-1-massachusetts-resident-income-tax-return/download"
    defined_for = StateCode.MA
    defined_for = StateCode.MA

    def formula(tax_unit, period, parameters):
        income_tax_before_credits = tax_unit(
            "ma_income_tax_before_credits", period
        )
        p = parameters(period).gov.states.ma.tax.income
        credit_value = add(tax_unit, period, p.credits.allowed)
        return income_tax_before_credits - credit_value



from openfisca_us.model_api import *


class employee_payroll_tax(Variable):
    value_type = float
    entity = TaxUnit
    label = "Employee payroll tax"
    definition_period = YEAR
    unit = USD

    def formula(tax_unit, period, parameters):
        PERSON_ELEMENTS = [
            "employee_social_security_tax",
            "employee_medicare_tax",
        ]
        TAX_UNIT_ELEMENTS = ["additional_medicare_tax"]
        person_elements = aggr(tax_unit, period, PERSON_ELEMENTS)
        tax_unit_elements = add(tax_unit, period, TAX_UNIT_ELEMENTS)
        return person_elements + tax_unit_elements



from openfisca_us.model_api import *


class qualified_energy_efficiency_improvements_expenditures(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    documentation = "Expenditures on qualified energy efficiency improvements"
    unit = USD
    reference = "https://www.law.cornell.edu/uscode/text/26/25C#c"

    formula = sum_of_variables(
        [
            "energy_efficient_door_expenditures",
            "energy_efficient_insulation_expenditures",
            "energy_efficient_roof_expenditures",
            "energy_efficient_window_expenditures",
        ]
    )



from openfisca_us.model_api import *


class biomass_fuel_property_expenditures(Variable):
    value_type = float
    entity = TaxUnit
    label = "Qualified biomass fuel property expenditures"
    documentation = "Expenditures for property—(i)which uses the burning of biomass fuel to heat a dwelling unit located in the United States and used as a residence by the taxpayer, or to heat water for use in such a dwelling unit, and (ii)which has a thermal efficiency rating of at least 75 percent (measured by the higher heating value of the fuel)."
    unit = USD
    definition_period = YEAR
    reference = "https://www.law.cornell.edu/uscode/text/26/25D#d_6"



from openfisca_us.model_api import *
import yaml


class medicaid_rating_area(Variable):
    value_type = int
    entity = Household
    label = "Medicaid rating area"
    definition_period = YEAR

    def formula(household, period, parameters):
        parameter_tree = household.simulation.tax_benefit_system.parameters
        mra = parameter_tree(
            period
        ).gov.hhs.medicaid.geography.medicaid_rating_area
        three_digit_zip_code = household("three_digit_zip_code", period)
        county = household("county_str", period)
        locations = np.array(list(mra._children))
        county_in_locations = np.isin(county, locations)
        location = where(
            county_in_locations,
            county,
            three_digit_zip_code,
        )
        valid_location = np.isin(location, locations)
        rating_areas = np.ones_like(
            location
        )  # ~2.8% of locations don't match with a a scraped MRA. For these, we assign to the State's first MRA.
        if valid_location.sum() > 0:
            rating_areas[valid_location] = mra[location[valid_location]]
        return rating_areas



from openfisca_us.model_api import *


class md_ctc(Variable):
    value_type = float
    entity = TaxUnit
    label = "MD CTC"
    definition_period = YEAR
    unit = USD
    documentation = "Maryland Child Tax Credit"
    reference = "https://casetext.com/statute/code-of-maryland/article-tax-general/title-10-income-tax/subtitle-7-income-tax-credits/section-10-751-effective-until-712026-tax-credit-for-qualified-child"
    defined_for = StateCode.MD

    def formula(tax_unit, period, parameters):
        p = parameters(period).gov.states.md.tax.income.credits.ctc
        income_eligible = (
            tax_unit("adjusted_gross_income", period) <= p.agi_cap
        )
        person = tax_unit.members
        dependent = person("is_tax_unit_dependent", period)
        disabled = person("is_disabled", period)
        meets_age_limit = person("age", period) < p.age_limit
        eligible_child = dependent & disabled & meets_age_limit
        eligible_children = tax_unit.sum(eligible_child)
        return income_eligible * eligible_children * p.amount



from openfisca_us.model_api import *


class tax_unit_itemizes(Variable):
    value_type = bool
    entity = TaxUnit
    label = "Itemizes tax deductions"
    unit = USD
    documentation = "Whether this tax unit elects to itemize deductions, rather than claim standard deductions."
    definition_period = YEAR

    def formula(tax_unit, period, parameters):
        tax_if_itemizing = tax_unit("tax_liability_if_itemizing", period)
        tax_if_not_itemizing = tax_unit(
            "tax_liability_if_not_itemizing", period
        )
        return tax_if_itemizing < tax_if_not_itemizing



from openfisca_us.model_api import *


class short_term_capital_gains(Variable):
    value_type = float
    entity = Person
    label = "Short-term capital gains"
    unit = USD
    documentation = "The sum of all gains made from (profitable) sales of assets held for one year or less."
    definition_period = YEAR
    reference = dict(
        title="26 U.S. Code § 1222(1)",
        href="https://www.law.cornell.edu/uscode/text/26/1222#1",
    )



from openfisca_us.model_api import *
import numpy as np


class new_electric_vehicle_credit(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    label = "New electric vehicle credit"
    documentation = (
        "Nonrefundable credit for the purchase of a new electric vehicle"
    )
    unit = USD
    reference = "https://www.law.cornell.edu/uscode/text/26/30D"
    defined_for = "new_electric_vehicle_credit_eligible"

    def formula(tax_unit, period, parameters):
        ira = parameters(
            period
        ).contrib.congress.senate.democrats.inflation_reduction_act
        # The Inflation Reduction Act changes the credit structure.
        if ira.in_effect:
            p = ira.electric_vehicle_credit.new
            battery_component_percent = tax_unit(
                "new_electric_vehicle_battery_components_made_in_north_america",
                period,
            )
            meets_battery_component_test = (
                battery_component_percent >= p.battery_components.threshold
            )
            critical_minerals_percent = tax_unit(
                "new_electric_vehicle_battery_critical_minerals_extracted_in_trading_partner_country",
                period,
            )
            meets_critical_minerals_test = (
                critical_minerals_percent >= p.critical_minerals.threshold
            )
            battery_components_credit = (
                meets_battery_component_test * p.battery_components.amount
            )
            critical_minerals_credit = (
                meets_critical_minerals_test * p.critical_minerals.amount
            )
            return battery_components_credit + critical_minerals_credit
        # Otherwise compute under current law.
        p = parameters(period).gov.irs.credits.electric_vehicle.new
        # Amount per kWh of EV battery capacity.
        # "In the case of a vehicle which draws propulsion energy from a
        # battery with not less than 5 kilowatt hours of capacity, the amount
        # determined under this paragraph is $417, plus $417 for each kilowatt
        # hour of capacity in excess of 5 kilowatt hours."
        capacity = tax_unit("new_electric_vehicle_battery_capacity", period)
        kwh_excess = max_(capacity - p.capacity_bonus.kwh_threshold + 1, 0)
        kwh_bonus = min_(
            p.capacity_bonus.amount * np.floor(kwh_excess),
            p.capacity_bonus.max,
        )
        return p.base_amount + kwh_bonus



from openfisca_us.model_api import *


class in_add_backs(Variable):
    value_type = float
    entity = TaxUnit
    label = "IN add-backs"
    unit = USD
    definition_period = YEAR
    reference = (
        "http://iga.in.gov/legislative/laws/2021/ic/titles/006#6-3-1-3.5"
    )

    formula = sum_of_variables(
        [
            "in_bonus_depreciation_add_back",
            "in_nol_add_back",
            "in_oos_municipal_obligation_interest_add_back",
            "in_other_add_backs",
            "in_section_179_expense_add_back",
            "in_tax_add_back",
        ]
    )



from openfisca_us.model_api import *


class qbid_limit(Variable):
    value_type = float
    entity = TaxUnit
    label = "Qualified business income deduction limit"
    unit = USD
    definition_period = YEAR
    reference = "https://www.law.cornell.edu/uscode/text/26/199A#b_1"

    def formula(tax_unit, period, parameters):
        qbi = add(tax_unit, period, ["qualified_business_income"])
        w2_wages = add(tax_unit, period, ["w2_wages_from_qualified_business"])
        qualified_property = add(
            tax_unit, period, ["unadjusted_basis_qualified_property"]
        )
        taxable_income_less_qbid = tax_unit("taxable_income_less_qbid", period)
        qbid = parameters(period).gov.irs.deductions.qbi
        filing_status = tax_unit("filing_status", period)
        phase_out_start = qbid.phase_out.start[filing_status]
        phase_out_length = qbid.phase_out.length[filing_status]
        capped_phase_out_income = min_(
            phase_out_length,
            max_(0, taxable_income_less_qbid - phase_out_start),
        )
        percent_through_phase_out = capped_phase_out_income / phase_out_length
        income_based_deduction = qbid.max.rate * qbi
        w2_and_property_based_deduction = (
            qbid.max.w2_wages.alt_rate * w2_wages
            + qbid.max.business_property.rate * qualified_property
        )
        reduction = percent_through_phase_out * max_(
            0, income_based_deduction - w2_and_property_based_deduction
        )
        return tax_unit("maximum_qbid", period) - reduction



from openfisca_us.model_api import *


class income_tax_non_refundable_credits(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    label = "Income tax non-refundable credits"
    documentation = (
        "Total non-refundable credits used to reduce positive tax liability"
    )
    unit = USD

    formula = sum_of_variables("gov.irs.credits.non_refundable")


c07100 = variable_alias("c07100", income_tax_non_refundable_credits)



from openfisca_us.model_api import *


class ny_itemized_deductions_max(Variable):
    value_type = float
    entity = TaxUnit
    label = "NY uncapped itemized deductions"
    unit = USD
    definition_period = YEAR
    reference = "https://www.nysenate.gov/legislation/laws/TAX/615"
    defined_for = StateCode.NY

    def formula(tax_unit, period, parameters):
        gov = parameters(period).gov
        federal_deductions_if_itemizing = (
            gov.irs.deductions.deductions_if_itemizing
        )
        federal_deductions_if_itemizing = [
            deduction
            for deduction in federal_deductions_if_itemizing
            if deduction
            not in [
                "salt_deduction",
                "qualified_business_income_deduction",
            ]
        ]
        return add(tax_unit, period, federal_deductions_if_itemizing)



from openfisca_us.model_api import *


class ny_income_tax(Variable):
    value_type = float
    entity = TaxUnit
    label = "NY income tax"
    unit = USD
    documentation = "Description"
    definition_period = YEAR
    defined_for = StateCode.NY

    def formula(tax_unit, period, parameters):
        income_tax = tax_unit("ny_income_tax_before_credits", period)
        non_refundable_credits = tax_unit("ny_non_refundable_credits", period)
        refundable = parameters(
            period
        ).gov.states.ny.tax.income.credits.refundable
        refundable_credits = add(tax_unit, period, refundable)
        return income_tax - non_refundable_credits - refundable_credits



from openfisca_us.model_api import *


class tax_unit_capital_loss(Variable):
    value_type = float
    entity = TaxUnit
    label = "Tax unit capital loss"
    unit = USD
    documentation = "Total capital losses for the tax unit"
    definition_period = YEAR
    reference = "https://www.law.cornell.edu/uscode/text/26/1211"

    formula = sum_of_variables(["capital_loss"])



from openfisca_us.model_api import *


class social_security(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    documentation = "Social Security benefits, not including SSI"
    label = "Social Security"
    unit = USD

    def formula(person, period, parameters):
        disabled_programs = parameters(period).simulation.disabled_programs
        if "social_security" in disabled_programs:
            return person("social_security_reported", period)
        SS_TYPES = ["dependents", "disability", "retirement", "survivors"]
        return add(person, period, ["social_security_" + i for i in SS_TYPES])



from openfisca_us.model_api import *


class self_employment_income(Variable):
    value_type = float
    entity = Person
    label = "Self-employment income"
    unit = USD
    documentation = "Self-employment non-farm income."
    definition_period = YEAR



from openfisca_us.model_api import *


class net_capital_gain(Variable):
    value_type = float
    entity = TaxUnit
    label = "Net capital gain"
    unit = USD
    documentation = 'The excess of net long-term capital gain over net short-term capital loss, plus qualified dividends (the definition of "net capital gain" which applies to 26 U.S.C. § 1(h) from § 1(h)(11)).'
    definition_period = YEAR
    reference = dict(
        title="26 U.S. Code § 1222(11)",
        href="https://www.law.cornell.edu/uscode/text/26/1222#11",
    )

    def formula(tax_unit, period, parameters):
        net_long_term_capital_gain = tax_unit(
            "net_long_term_capital_gain", period
        )
        net_short_term_capital_loss = tax_unit(
            "net_short_term_capital_loss", period
        )
        net_capital_gains = max_(
            0, net_long_term_capital_gain - net_short_term_capital_loss
        )
        qualified_dividends = add(
            tax_unit, period, ["qualified_dividend_income"]
        )
        return net_capital_gains + qualified_dividends



from openfisca_us.model_api import *


class spm_unit_benefits(Variable):
    value_type = float
    entity = SPMUnit
    label = "Benefits"
    definition_period = YEAR
    unit = USD

    formula = sum_of_variables(
        [
            "social_security",
            "ssi",
            "ca_cvrp",  # California Clean Vehicle Rebate Project.
            "snap",
            "wic",
            "free_school_meals",
            "reduced_price_school_meals",
            "lifeline",
            "acp",
            "ebb",
            "spm_unit_capped_housing_subsidy",
            "tanf",
            # Contributed.
            "basic_income",
        ]
    )



from openfisca_tools import Microsimulation as GeneralMicrosimulation
from openfisca_us import CountryTaxBenefitSystem
from openfisca_us.entities import entities
from openfisca_us.data import CPS
from openfisca_tools.data import Dataset


class Microsimulation(GeneralMicrosimulation):
    tax_benefit_system = CountryTaxBenefitSystem
    entities = entities
    default_dataset = CPS

    def __init__(
        self, reform=(), dataset: type = CPS, year: int = None, **kwargs
    ):
        if dataset == CPS and len(CPS.years) == 0:
            CPS.generate(2020)

        super().__init__(reform, dataset, year)

        self.year = 2022



from openfisca_us.model_api import *


class in_renters_deduction(Variable):
    value_type = float
    entity = TaxUnit
    label = "IN renter's deductions"
    unit = USD
    definition_period = YEAR
    reference = "http://iga.in.gov/legislative/laws/2021/ic/titles/006#6-3-2-6"

    def formula(tax_unit, period, parameters):
        p = parameters(period).gov.states["in"].tax.income.deductions
        filing_status = tax_unit("filing_status", period)
        max_renters_deduction = p.renters.max[filing_status]
        in_rent = add(tax_unit, period, ["rent"])
        # using national rent var to save mem but law specifices only IN rent allowed
        return min_(in_rent, max_renters_deduction)



from openfisca_us.model_api import *


class eitc_eligible(Variable):
    value_type = bool
    entity = TaxUnit
    label = "Eligible for EITC"
    definition_period = YEAR
    reference = "https://www.law.cornell.edu/uscode/text/26/32#c_1_A"

    def formula(tax_unit, period, parameters):
        person = tax_unit.members
        has_child = tax_unit.any(person("is_child", period))
        age = person("age", period)
        eitc = parameters(period).gov.irs.credits.eitc
        meets_age_requirements = (age >= eitc.eligibility.age.min) & (
            age <= eitc.eligibility.age.max
        )
        inv_income_sources = eitc.eligibility.counted_inv_income
        investment_income = (
            aggr(tax_unit, period, inv_income_sources)
            + max_(0, tax_unit("c01000", period))
            + max_(
                0,
                tax_unit("tax_unit_rental_income", period)
                - tax_unit("tax_unit_partnership_s_corp_income", period),
            )
        )
        inv_income_disqualified = (
            investment_income > eitc.phase_out.max_investment_income
        )
        eligible = has_child | tax_unit.any(meets_age_requirements)
        return eligible & ~inv_income_disqualified



from openfisca_us.model_api import *


class md_income_tax(Variable):
    value_type = float
    entity = TaxUnit
    label = "MD income tax"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.MD

    def formula(tax_unit, period, parameters):
        in_md = tax_unit.household("state_code_str", period) == "MD"
        tax_after_non_refundable_credits = tax_unit(
            "md_income_tax_after_non_refundable_credits", period
        )
        refundable_credits = tax_unit("md_refundable_credits", period)
        return in_md * (tax_after_non_refundable_credits - refundable_credits)



from openfisca_us.model_api import *


class pension_income(Variable):
    value_type = float
    entity = Person
    label = "Pension income"
    unit = USD
    documentation = "Income from pensions, annuitities, life insurance or endowment contracts."
    definition_period = YEAR

    formula = sum_of_variables(
        ["tax_exempt_pension_income", "taxable_pension_income"]
    )



from openfisca_us.model_api import *


class is_ccdf_initial_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    definition_period = YEAR
    label = "Initial income eligibility for CCDF"



from openfisca_us.model_api import *


class farm_income(Variable):
    value_type = float
    entity = Person
    label = "Farm income"
    unit = USD
    documentation = "Income from agricultural businesses. Do not include this income in self-employment income."
    definition_period = YEAR



"""This file contains your country package's metadata and dependencies."""

from setuptools import find_packages, setup

setup(
    name="OpenFisca-US",
    version="0.129.0",
    author="PolicyEngine",
    author_email="hello@policyengine.org",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: POSIX",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
        "Topic :: Scientific/Engineering :: Information Analysis",
    ],
    description="OpenFisca tax and benefit system for the US",
    keywords="benefit microsimulation social tax",
    license="http://www.fsf.org/licensing/licenses/agpl-3.0.html",
    url="https://github.com/PolicyEngine/openfisca-us",
    include_package_data=True,  # Will read MANIFEST.in
    data_files=[
        (
            "share/openfisca/openfisca-country-template",
            ["CHANGELOG.md", "LICENSE", "README.md"],
        ),
    ],
    install_requires=[
        "h5py",
        "microdf_python",
        "OpenFisca-Core[web-api] >= 35.0.0",
        "OpenFisca-Tools>=0.13.3,<1.0.0",
        "pandas",
        "pathlib",
        "pytest",
        "pytest-dependency",
        "pyyaml",
        "requests",
        "synthimpute",
        "tables",
        "tqdm",
        "click>=8.0.0",
    ],
    extras_require={
        "dev": [
            "autopep8 >= 1.5.4, < 2.0.0",
            "black",
            "coverage",
            "jupyter-book",
            "plotly",
            "pytest",
            "setuptools",
            "wheel",
            "yaml-changelog>=0.1.7",
            "linecheck",
        ],
    },
    python_requires=">=3.7,<3.8",
    entry_points={
        "console_scripts": [
            "openfisca-us = openfisca_us.tools.cli:main",
            "openfisca-us-data = openfisca_us.data.cli:cli",
        ],
    },
    packages=find_packages(),
)



from openfisca_us.model_api import *


class spm_unit_net_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Net income"
    definition_period = YEAR
    unit = USD

    def formula(spm_unit, period, parameters):
        reported_net_income = spm_unit("spm_unit_net_income_reported", period)
        if reported_net_income.sum() > 0:
            # If we have reported net income, use that instead for now. This
            # is only until the full microsimulation can be run.
            return reported_net_income
        market_income = spm_unit("spm_unit_market_income", period)
        benefits = spm_unit("spm_unit_benefits", period)
        taxes = spm_unit("spm_unit_taxes", period)
        return market_income + benefits - taxes



from openfisca_us.model_api import *


class tax_exempt_unemployment_compensation(Variable):
    value_type = float
    entity = Person
    label = "Tax-exempt unemployment compensation"
    unit = USD
    definition_period = YEAR

    def formula(person, period, parameters):
        total_ui = person("unemployment_compensation", period)
        taxable_ui = person("taxable_unemployment_compensation", period)
        return total_ui - taxable_ui



from openfisca_us.model_api import *


class business_income(Variable):
    value_type = float
    entity = Person
    label = "Business income (losses ignored)"
    unit = USD
    documentation = "Business income, capped at zero."
    definition_period = YEAR

    def formula(person, period, parameters):
        profit = person("self_employment_income", period)
        rents = person("rental_income", period)
        positive_profit = max_(profit, 0)
        positive_rents = max_(rents, 0)
        return positive_profit + positive_rents



from openfisca_us.model_api import *


class prior_nonbusiness_energy_property_credits(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    documentation = (
        "Nonbusiness energy property credits claimed in prior years"
    )
    unit = USD
    reference = "https://www.law.cornell.edu/uscode/text/26/25C#b_1"



from openfisca_us.model_api import *


class capital_loss(Variable):
    value_type = float
    entity = Person
    label = "Capital loss"
    unit = USD
    documentation = "Losses from transactions involving property."
    definition_period = YEAR

    def formula(tax_unit, period, parameters):
        capital_gains = tax_unit("capital_gains", period)
        return max_(0, -capital_gains)



from openfisca_us.model_api import *


class ny_itemized_deductions(Variable):
    value_type = float
    entity = TaxUnit
    label = "NY itemized deductions"
    unit = USD
    definition_period = YEAR
    reference = "https://www.nysenate.gov/legislation/laws/TAX/615"
    defined_for = StateCode.NY

    def formula(tax_unit, period, parameters):
        max_deductions = tax_unit("ny_itemized_deductions_max", period)
        reduction = tax_unit("ny_itemized_deductions_reduction", period)
        # There are some other specific details about some types of itemized deductions
        # likely non-modellable in the CPS. Requires further investigation.
        return max_deductions - reduction
